package com.lantone.qc.kernel.util;


import com.lantone.qc.nlp.participle.cfg.Configuration;
import com.lantone.qc.nlp.participle.cfg.DefaultConfig;
import com.lantone.qc.security.util.EncrypDES;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName : CacheUtil
 * @Description : Redis存放
 * @Author : 楼辉荣
 * @Date: 2020-03-11 11:42
 */
@Component
public class CacheUtil implements ApplicationRunner {
    Logger logger = LoggerFactory.getLogger(DefaultConfig.class);
    @Autowired
    private RedisUtil redisUtil;

    public void run(ApplicationArguments var1) {
        try {
            this.putConceptDiagPropertyMap();//疾病相关信息放入缓存 包括疾病是否是慢病、传染病、常见症状、同义词
            this.putConceptPropMap();//性质词典
            this.putConceptDegreeMap();//程度词典
            this.putDiagHospitalReflect();//医院疾病名称映射，仅做测试用，由华卓完成和医院对接放入redis
            this.putClinicBodyPartMap();//症状需要部位描述词典放入redis缓存
        } catch (Exception e) {
            logger.error("缓存文件读取错误。。。");
        }
    }

    /**
     * 疾病相关信息放入缓存
     * 包括疾病是否是慢病、传染病、常见症状、同义词
     */
    public void putConceptDiagPropertyMap() throws Exception {
        Configuration configuration = new DefaultConfig();
        List<String> lines = configuration.readTargetFileContents("cache/concept_diag_properties.dict");

        Map<String, Map<String, Object>> diagMap = new HashMap<>();
        for (String line : lines) {
            String[] line_arr = line.split("\\|", -1);
            if (line_arr.length == 5) {
                List<String> synonymsList = new ArrayList<>();
                List<String> symptomsList = new ArrayList<>();
                if (StringUtils.isNotEmpty(line_arr[3])) {
                    synonymsList = java.util.Arrays.asList(line_arr[3].split(","));
                }
                if (StringUtils.isNotEmpty(line_arr[4])) {
                    symptomsList = java.util.Arrays.asList(line_arr[4].split(","));
                }
                Map<String, Object> map = new HashMap<>();
                map.put("infectious", line_arr[2]);
                map.put("chronic", line_arr[1]);
                map.put("synonyms", synonymsList);
                map.put("symptoms", symptomsList);
                diagMap.put(line_arr[0], map);
            }
        }
        redisUtil.set(KernelConstants.CONCEPT_DIAG_PROPERTY_MAP, diagMap);
    }

    /**
     * 性质词典放入redis缓存
     */
    public void putConceptPropMap() throws Exception {
        Configuration configuration = new DefaultConfig();
        List<String> lines = configuration.readTargetFileContents("cache/concept_prop.dict");
        List<String> propList = new ArrayList<>();
        for (String line : lines) {
            propList.add(line);
        }
        redisUtil.set(KernelConstants.CONCEPT_PROPERTY_LIST, propList);
    }

    /**
     * 程度词典放入redis缓存
     */
    public void putConceptDegreeMap() throws Exception {
        Configuration configuration = new DefaultConfig();
        List<String> lines = configuration.readTargetFileContents("cache/concept_degree.dict");
        List<String> degreeList = new ArrayList<>();
        for (String line : lines) {
            degreeList.add(line);
        }
        redisUtil.set(KernelConstants.CONCEPT_DEGREE_LIST, degreeList);
    }

    /**
     * 症状需要部位描述词典放入redis缓存
     */
    public void putClinicBodyPartMap() throws Exception {
        Configuration configuration = new DefaultConfig();
        List<String> lines = configuration.readTargetFileContents("cache/concept_clinic_bodypart_properties.dict");
        List<String> clinicBodyPartList = new ArrayList<>();
        for (String line : lines) {
            clinicBodyPartList.add(line);
        }
        redisUtil.set(KernelConstants.CONCEPT_CLINIC_BODYPART_PROPERTIES_LIST, clinicBodyPartList);
    }

    /**
     * 医院疾病名称映射，仅做测试用，由华卓完成和医院对接放入redis
     * @throws Exception
     */
    public void putDiagHospitalReflect() throws Exception {
        Map<String, String> map = new HashMap<>();
        map.put("腹痛待查", "腹痛待查 Concept");
        redisUtil.set(KernelConstants.CONCEPT_DIAG_HOSPITAL_REFLECT, map);
    }
}