package com.lantone.qc.kernel.web.controller;

import com.google.common.collect.Lists;
import com.lantone.qc.kernel.analysis.QCAnalysis;
import com.lantone.qc.kernel.util.CacheFileManager;
import com.lantone.qc.kernel.util.KernelConstants;
import com.lantone.qc.pub.jdbc.MysqlJdbc;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.vo.MedrecVo;
import com.lantone.qc.pub.model.vo.QueryVo;
import com.lantone.qc.pub.res.Response;
import com.lantone.qc.pub.util.PropertiesUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName : QCTestController
 * @Description : 测试
 * @Author : 楼辉荣
 * @Date: 2020-03-11 16:03
 */
@Api(value = "质控测试接口", tags = { "质控测试接口" })
@RestController
@RequestMapping(value = "test")
public class QCTestController {
    @Autowired
    private QCAnalysis qCAnalysis;

    @ApiOperation(value = "质控测试接口，无需token信息", notes = "")
    @PostMapping("rec_test")
    public Response<OutputInfo> extract(String caseIds, int length, String cid, String txtId) {
        Response response = new Response();
        PropertiesUtil propertiesUtil = new PropertiesUtil("kernel.properties");

        MysqlJdbc mysqlJdbc = new MysqlJdbc(propertiesUtil.getProperty("mysql.test.user"),
                propertiesUtil.getProperty("mysql.test.password"),
                propertiesUtil.getProperty("mysql.test.url"));
        Connection conn = mysqlJdbc.connect();
        Statement st = null;
        ResultSet rs = null;
        String r1, r2, r3, r4, r5;
        List<Map<String, Object>> updates = new ArrayList<>();
        List<Map<String, Object>> wheres = new ArrayList<>();
        try {
            String textSql = "select id, text from qc_inputcases ";
            if (StringUtils.isNotEmpty(txtId)) {
                textSql = textSql + " where id >= " + txtId;
            }
            textSql = textSql + " order by id limit 0, " + length;
            //更新pass状态
            mysqlJdbc.update("update qc_inputcases_mapping_all qim, " +
                    "(" + textSql + ") qi\n" +
                    "set qim.pass = 0 where qim.text_id = qi.id");

            st = conn.createStatement();
            //测试数据查询
            String sql = "SELECT qmap.id, qi.id text_id, qi.text, ca.name, entry.code FROM " +
                    "qc_inputcases_mapping_all qmap, (" + textSql + ") qi, qc_cases_entry entry, qc_cases ca " +
                    "where qmap.text_id = qi.id and qmap.cases_entry_id = entry.id and entry.cases_id = ca.id";

            if (StringUtils.isNotEmpty(caseIds)) {
                sql = sql + " and entry.cases_id in (" + caseIds + ")";
            }

            sql = sql + " order by qi.id, entry.code";

            Map<String, String> map = new HashMap<>();
            rs = st.executeQuery(sql);
            //条目数据结构， Key = 条目Code value = 条目属性{"precode":"BEH0002,BEH0003","title":"症状缺少时间描述"}
            Map<String, Map<String, String>> catalogueMap = new HashMap<>();
            //测试条目编码和qc_inputcases_mapping_all 的id，为了更新每个条目是否验证通过
            Map<String, String> updateMap = new HashMap<>();

            List<Map<String, Map<String, String>>> catalogueList = new ArrayList<>();
            List<String> textList = new ArrayList<>();
            List<String> caseList = new ArrayList<>();
            List<Map<String, String>> updateList = new ArrayList<>();
            List<String> textIdList = new ArrayList<>();
            String textId = ""; int index = 0;
            String text = ""; String caseName = "";
            while (rs.next()) {
                r1 = rs.getString(1);
                r2 = rs.getString(2);
                r3 = rs.getString(3);
                r4 = rs.getString(4);
                r5 = rs.getString(5);
                if (!textId.equals(String.valueOf(r2))) {
                    if (StringUtils.isNotEmpty(textId)) {
                        catalogueList.add(catalogueMap);
                        updateList.add(updateMap);
                        textList.add(text);
                        caseList.add(caseName);
                        textIdList.add(textId);

                        catalogueMap = new HashMap<>();
                        updateMap = new HashMap<>();
                    }
                    catalogueMap = new HashMap<>();
                }
                catalogueMap.put(r5, new HashMap<>());
                updateMap.put(r5, r1);
                textId = r2;
                text = r3;
                caseName = r4;
            }
            //最后一份病历无法在循环中写入 需要填回
            if (StringUtils.isNotEmpty(textId)) {
                catalogueList.add(catalogueMap);
                updateList.add(updateMap);
                textList.add(text);
                caseList.add(caseName);
                textIdList.add(textId);
            }

            for (int i = 0; i < textList.size(); i++) {
                System.out.println("**************************" + textIdList.get(i));
                MedrecVo medrecVo = new MedrecVo();
                medrecVo.setTitle(caseList.get(i));
                Map<String, Object> content = new HashMap<>();
                content.put("content", textList.get(i));
                medrecVo.setContent(content);
                if (caseList.get(i).equals("入院记录")){
                    medrecVo.setLabel(Lists.newArrayList("姓名", "性别", "年龄", "民族", "职业", "出生地", "婚姻", "联系地址", "病史陈述者", "出生日期", "户口地址", "电话", "入院日期", "记录日期", "主诉", "现病史", "既往史", "个人史", "婚育史", "月经史", "家族史", "体格检查（一）", "体格检查（二）", "辅助检查", "初步诊断", "修正诊断", "医师签名", "补充诊断"));
                }


                List<MedrecVo> medrec = new ArrayList<>();
                medrec.add(medrecVo);

                QueryVo queryVo = new QueryVo();
                queryVo.setMedrec(medrec);
                queryVo.setCid(cid);
                queryVo.setInputCatalogueMap(catalogueList.get(i));

                OutputInfo outputInfo = qCAnalysis.anlysis(queryVo);

                Map<String, String> catalogue_map = updateList.get(i);
                for (Map.Entry<String, String> entry : catalogue_map.entrySet()) {
                    Map<String, Object> m = new HashMap<>();
                    Map<String, Object> w = new HashMap<>();
                    if (outputInfo.getResult().get(entry.getKey()) == null) {
                        m.put("pass", "-2");
                    } else {
                        m.put("pass", outputInfo.getResult().get(entry.getKey()).get("status"));
                    }
                    w.put("id", catalogue_map.get(entry.getKey()));

                    updates.add(m);
                    wheres.add(w);
                }
            }


            mysqlJdbc.update("qc_inputcases_mapping_all", updates, wheres);



            CacheFileManager cacheFileManager = new CacheFileManager();
            cacheFileManager.insertFeature(KernelConstants.clinicModificationList, "kl_modification", "name");
            cacheFileManager.insertFeature(KernelConstants.clinicNameList, "kl_clinic", "name");
            cacheFileManager.insertFeature(KernelConstants.hospitalDiagNameList, "kl_disease", "name");

        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            mysqlJdbc.close(rs, st, conn);
        }
        return response;
    }
}